#include <stdio.h>
#include <unistd.h>
#include <wait.h>
#include <sys/types.h>
#include <signal.h>
#include "disco.h"

int main(int argc, char **argv) {
  int ret, pid, skipsnmp, sockfd;
  char *hostname, *filename;
  char writecom[1024];

  argv++;
  argc--;

  hostname=NULL;
  filename=NULL;
  strcpy(writecom, "");
  skipsnmp=0;
  verbose=0;
  while (argc) {
    if (!strcmp(argv[0], "-n")) {
      skipsnmp=1;
      argc--;
      argv++;
    } else if (!strcmp(argv[0], "-v")) {
      printf("Verbose mode\n");
      verbose=1;
      argc--;
      argv++;
    } else if (!strcmp(argv[0], "-w")) {
      if (argc < 2) {
	usage();
	exit(1);
      }
      strcpy(writecom, argv[1]);
      argc-=2;
      argv+=2;
      continue;
    } else if (!hostname) {
      hostname=argv[0];
      argc--;
      argv++;
    } else if (!filename) {
      filename=argv[0];
      argc--;
      argv++;
    } else {
      usage();
      exit(1);
    }
  }

  if (!hostname) {
    usage();
    exit(1);
  }
  if (!filename) filename=hostname;

  /* ask for the write community if it wasn't set on the command line */
  if (!strcmp(writecom, "")) {
    ret=get_snmp_writecom(writecom, 1024);
    if (ret) {
      printf("Error getting write community.\n");
      exit(1);
    }
    if (verbose) printf("Write community is %i chars long, stargs with '%c'\n", strlen(writecom), writecom[0]);
  }

  /* start the tftp server */
  sockfd=tftp_setup();
  if (sockfd==-1) {
    printf("Transfer aborted.\n");
    exit(1);
  }

  /* send the SNMP packet */
  if (!skipsnmp) {
    ret=snmp_get_config(hostname, writecom);
    if (ret) {
      printf("No response to SNMP, transfer aborted.\n");
      exit(1);
    }
  } else {
    if (verbose) printf("Skipping SNMP\n");
  }

  /* listen for data */
  ret=tftp_server(sockfd, filename, hostname);
  if (ret) {
    printf("Transfer aborted.\n");
    exit(1);
  }

  exit(0);
}

void usage() {
  printf("disco version 1.1\n");
  printf("usage: disco [-n] [-v] host [ file ] [-w community ]\n");
  printf("    host       hostname of the router to query\n");
  printf("    file       file to write to (default to hostname)\n");
  printf("    community  specify the SNMP write community (prompted for if not supplied)\n");
  printf("    -n         don't send the SNMP packet to the router that sends the config.\n");
  printf("    -v         verbose debugging.\n");
}
